package DebPool::Dirs;

###
#
# DebPool::Dirs - Module for dealing with directory related tasks
#
# Copyright (c) 2003 Joel Baker. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the Author nor the names of any contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Dirs.pm,v 1.3 2003/10/06 22:12:56 lucifer Exp $
#
###

# We use 'our', so we must have at least Perl 5.6

require v5.6.0;

# Always good ideas.

use strict;
use warnings;

### Module setup

BEGIN {
    use Exporter ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # Version checking
    $VERSION = "0.1.0";

    @ISA = qw(Exporter);

    @EXPORT = qw(
    );

    @EXPORT_OK = qw(
        &Create_Tree
        &Monitor_Incoming
        &PoolDir
        &Scan_Changes
        &Scan_All
    );

    %EXPORT_TAGS = (
        'functions' => [qw(&Create_Tree &Monitor_Incoming &PoolDir
                           &Scan_Changes &Scan_All)],
        'vars' => [qw()],
    );
}

### Exported package globals

# None

### Non-exported package globals

# Thread-safe? What's that? Package global error value. We don't export
# this directly, because it would conflict with other modules.

our($Error);

### File lexicals

# None

### Constant functions

# None

### Meaningful functions

# Create_Tree()
#
# Creates a full directory tree based on the current directory values in
# %DebPool::Config::Options. Returns 1 on success, 0 on failure (and sets
# or propagates $Error).

sub Create_Tree {
    use DebPool::Config qw(:vars);

    # Basic directories - none of these are terribly exciting. We don't set
    # $Error on failure, because Tree_Mkdir will have already done so.

    if (!Tree_Mkdir($Options{'db_dir'}, $Options{'db_dir_mode'})) {
        return 0;
    }

    if (!Tree_Mkdir($Options{'incoming_dir'}, $Options{'incoming_dir_mode'})) {
        return 0;
    }

    if (!Tree_Mkdir($Options{'installed_dir'}, $Options{'installed_dir_mode'})) {
        return 0;
    }

    if (!Tree_Mkdir($Options{'reject_dir'}, $Options{'reject_dir_mode'})) {
        return 0;
    }

    # Now the distribution directory and subdirectories

    my($dists_dir) = $Options{'dists_dir'};
    my($dists_dir_mode) = $Options{'dists_dir_mode'};

    if (!Tree_Mkdir($dists_dir, $dists_dir_mode)) {
        return 0;
    }

    # Real distributions are the only ones that get directories.

    my($dist);
    foreach $dist (@{$Options{'realdists'}}) {
        if (!Tree_Mkdir("$dists_dir/$dist", $dists_dir_mode)) {
            return 0;
        }

        my($section);
        foreach $section (@{$Options{'sections'}}) {
            if (!Tree_Mkdir("$dists_dir/$dist/$section", $dists_dir_mode)) {
                return 0;
            }

            my($arch);
            foreach $arch (@{$Options{'archs'}}) {
                my($target) = "$dists_dir/$dist/$section/";
                if ('source' eq $arch) {
                    $target .= $arch;
                } else {
                    $target .= "binary-$arch";
                }

                if (!Tree_Mkdir($target, $dists_dir_mode)) {
                    return 0;
                }
            }
        }
    }

    # Go through all of the distributions looking for those that should be
    # symlinks, and creating them if necessary.

    foreach $dist (keys(%{$Options{'dists'}})) {
        # Check whether it should be a symlink. If so, make sure it is.

        if (!($dist eq $Options{'dists'}->{$dist})) { # Different names -> sym
            if (! -e "$dists_dir/$dist") {
                if (!symlink($Options{'dists'}->{$dist}, "$dists_dir/$dist")) {
                    $Error = "Couldn't create symlink $dists_dir/$dist -> ";
                    $Error .= "$Options{'dists'}->{$dist}: $!";
                }
            } elsif (! -l "$dists_dir/$dist") {
                $Error = "$dists_dir/$dist exists and isn't a symlink, ";
                $Error .= "but it should be";
                return 0;
            }
        }
    }

    # And, finally, the pool directory and subdirectories

    my($pool_dir) = $Options{'pool_dir'};
    my($pool_dir_mode) = $Options{'pool_dir_mode'};

    if (!Tree_Mkdir($pool_dir, $pool_dir_mode)) {
        return 0;
    }

    # We can only get away with this because Debian pool directories are
    # named in ASCII...

    my($letter);
    foreach $letter ('a' .. 'z') {
        if (!Tree_Mkdir("$pool_dir/$letter", $pool_dir_mode)) {
            return 0;
        }
        if (!Tree_Mkdir("$pool_dir/lib$letter", $pool_dir_mode)) {
            return 0;
        }
    }

    return 1;
}

# Tree_Mkdir($directory, $mode)
#
# Creates $directory with $mode. Returns 0 and sets $Error on failure, or
# 1 on success.
#
# Internal support function for Create_Tree

sub Tree_Mkdir {
    my($dir, $mode) = @_;

    if (-d $dir) {
        return 1;
    };

    if (-e $dir) {
        $Error = "Couldn't create '$dir' - already exists as a non-directory.";
        return 0;
    }

    if (!mkdir($dir, $mode)) {
        $Error = "Couldn't create '$dir': $!";
        return 0;
    }

    if (!chmod($mode, $dir)) {
        $Error = "Couldn't chmod '$dir': $!";
        return 0;
    }

    return 1;
}

# Scan_Changes($directory)
#
# Scan the specified directory for changes files. Returns an array of
# filenames relative to the directory, or undef (and sets $Error) on an error.

sub Scan_Changes {
    my($directory) = @_;

    if (!opendir(INCOMING, $directory)) {
        $Error = "Couldn't open directory '$directory': $!";
        return undef;
    }

    # Perl magic - read the directory and grep it for *.changes all at one
    # shot.

    my(@changes) = grep(/\.changes$/, readdir(INCOMING));
    close(INCOMING);

    return @changes;
}

# Scan_All($directory)
#
# Scans the specified directory and all subdirectories for any files.
# Returns an arrayref pointing to an array of filepaths relative to
# $directory, or undef (and sets $Error) on failure. Ignores any hidden
# files or directories.

sub Scan_All {
    my($directory) = @_;

    if (!opendir(DIR, $directory)) {
        $Error = "Couldn't open directory '$directory'";
        return undef;
    }

    my($direntry);
    my(@entries) = grep(!/^\./, readdir(DIR));

    my(@return);

    foreach $direntry (@entries) {
        if (-f "$directory/$direntry") {
            push(@return, $direntry);
        } elsif (-d "$directory/$direntry") {
            my($recurse) = Scan_All("$directory/$direntry");

            if (!defined($recurse)) { # $Error is already set.
                return undef;
            }

            # I'd like to use map(), but Perl makes stooopid guesses.

            my($entry);

            foreach $entry (@{$recurse}) {
                push(@return, "$direntry/$entry");
            }
        }
    }

    return \@return;
}

# Monitor_Incoming()
#
# Monitors the incoming directory, looping until the directory is updated.
# Returns 1 on success, 0 on failure (and sets $Error).

sub Monitor_Incoming {
    use DebPool::Config;

    # If this is ever false, we either shouldn't have been called in the
    # first place, or we've caught a signal and shouldn't do anything
    # further.

    if ($DebPool::Signal::Signal_Caught) {
        return 1;
    }

    my(@stat) = stat($Options{'incoming_dir'});
    if (!@stat) {
        $Error = "Couldn't stat incoming_dir '$Options{'incoming_dir'}'";
        return 0;
    }
    my($mtime) = $stat[9];

    do {
        sleep($Options{'sleep'});
        @stat = stat($Options{'incoming_dir'});
        if (!@stat) {
            $Error = "Couldn't stat incoming_dir '$Options{'incoming_dir'}'";
            return 0;
        }
    } until (($stat[9] != $mtime) || ($DebPool::Signal::Signal_Caught));

    return 1;
}

# PoolDir($name)
#
# Calculates a pool subdirectory name from the package name.

sub PoolDir {
    my($name) = @_;

    # Pool subdirectories are normally the first letter of the package
    # name, unless it is a lib* package, in which case the subdir is
    # lib<first letter>.

    if ($name =~ s/^lib//) { # lib(.).*
        return 'lib' . substr($name, 0, 1);
    } else { # (.).*
        return substr($name, 0, 1);
    }
}

END {}

1;

__END__

# vim:set tabstop=4 expandtab:
