package DebPool::Release;

###
#
# DebPool::Release - Module for generating and installing Release files
#
# Copyright (c) 2003 Joel Baker. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the Author nor the names of any contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Release.pm,v 1.2 2003/10/06 22:12:24 lucifer Exp $
#
###

# We use 'our', so we must have at least Perl 5.6

require v5.6.0;

# Always good ideas.

use strict;
use warnings;

use POSIX; # tmpnam()
use Fcntl;

# We need the Digest modules so that we can calculate the proper checksums.

use Digest::MD5;
use Digest::SHA1;

### Module setup

BEGIN {
    use Exporter ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # Version checking
    $VERSION = "0.1.0";

    @ISA = qw(Exporter);

    @EXPORT = qw(
    );

    @EXPORT_OK = qw(
        &Generate_Release
        &Install_Release
    );

    %EXPORT_TAGS = (
        'functions' => [qw(&Generate_Release &Install_Release)],
        'vars' => [qw()],
    );
}

### Exported package globals

### Non-exported package globals

# Thread-safe? What's that? Package global error value. We don't export
# this directly, because it would conflict with other modules.

our($Error);

# Magic filenames - these are files we want to include hashes for in a
# Release file.

my(@SigFiles) = (
    'Packages',
    'Sources',
    'Packages.gz',
    'Sources.gz',
);

### File lexicals

# None

### Constant functions

# None

### Meaningful functions

# Generate_Release($archive, $component, $architecture, $version)
#
# Generate a Release file in the temp/working area, and return the filename.
# Returns undef (and sets $Error) on error.

sub Generate_Release {
    use DebPool::Config qw(:vars);

    my($archive, $component, $architecture, $version) = @_;

    my(%Checksums);

    # Before we bother to do much else, generate the MD5 and SHA1 checksums
    # we'll need later. This is mostly so that we can catch errors before
    # ever bothering to open a tempfile.

    # First, grab a list of files from the directory.

    my($dirpath) = "$Options{dists_dir}/$archive/$component/";
    if ('source' eq $architecture) {
        $dirpath .= $architecture;
    } else {
        $dirpath .= "binary-$architecture";
    }

    if (!opendir(RELDIR, $dirpath)) {
        $Error = "Couldn't open directory '$dirpath'.";
        return undef;
    }

    my(@dirfiles) = readdir(RELDIR);
    close(RELDIR);

    # Now, for each file, generate MD5 and SHA1 checksums, and put them
    # into Checksums for later use (assuming it's a file we care about).

    my($ck_file);

    foreach $ck_file (@dirfiles) {
        if (0 == grep(/^$ck_file$/, @SigFiles)) { # We don't care about it.
            next;
        }

        # Grab the filesize from stat()

        my(@stat) = stat("$dirpath/$ck_file");
        my($size) = $stat[7];

        # Open the file and read in the contents. This could be a very
        # large amount of data, but unfortunately, both Digest routines
        # require the entire thing at once.

        if (!open(CK_FILE, '<', "$dirpath/$ck_file")) {
            $Error = "Couldn't open file '$ck_file' for reading.";
            return undef;
        }

        my(@filetext) = <CK_FILE>;
        close(CK_FILE);

        # Now calculate the checksums and put them into the hashes.

        my($md5) = Digest::MD5::md5_hex(@filetext);
        my($sha1) = Digest::SHA1::sha1_hex(@filetext);

        $Checksums{$ck_file} = {
            'Size' => $size,
            'MD5' => $md5,
            'SHA1' => $sha1,
        };
    }

    # Open a secure tempfile - we could loop on this, but it has the
    # potential for spinlock. So if it fails, we just error out.

    my($tmpfile) = tmpnam();
    if (!sysopen(RELEASE, $tmpfile, O_WRONLY|O_CREAT|O_EXCL, 0644)) {
        $Error = "Couldn't open secure tempfile.\n";
        return undef;
    }

    # Write the headers into the Release tempfile

    print RELEASE "Archive: $archive\n";
    print RELEASE "Component: $component\n";
    print RELEASE "Version: $version\n";
    print RELEASE "Origin: $Options{'release_origin'}\n";
    print RELEASE "Label: $Options{'release_label'}\n";
    print RELEASE "Architecture: $architecture\n";

    # If the archive (aka distribution) appears in release_noauto, print
    # the appropriate directive.

    if (0 != grep(/^$archive$/, @{$Options{'release_noauto'}})) {
        print RELEASE "NotAutomatic: yes\n";
    }

    print RELEASE "Description: $Options{'release_description'}\n";

    # Now print MD5 and SHA1 checksum lists.

    print RELEASE "MD5Sum:\n";
    foreach $ck_file (keys(%Checksums)) {
        printf RELEASE " %s %8d %s\n", $Checksums{$ck_file}->{'MD5'},
            $Checksums{$ck_file}->{'Size'}, $ck_file;
    }

    print RELEASE "SHA1:\n";
    foreach $ck_file (keys(%Checksums)) {
        printf RELEASE " %s %8d %s\n", $Checksums{$ck_file}->{'SHA1'},
            $Checksums{$ck_file}->{'Size'}, $ck_file;
    }

    close(RELEASE);

    return $tmpfile;
}

# Install_Release($archive, $component, $architecture, $release, $signature)
#
# Installs a release file and an optional signature file to the
# distribution directory specified by the ($archive, $component,
# $architecture) triple. Returns 0 (and sets $Error) on failure, 1 on
# success.

sub Install_Release {
    use DebPool::Config qw(:vars);

    my($archive, $component, $architecture, $release, $signature) = @_;

    my($dists_file_mode) = $Options{'dists_file_mode'};
    my($inst_dir) = "$Options{'dists_dir'}/$archive/$component/";

    if ('source' eq $architecture) {
        $inst_dir .= $architecture;
    } else {
        $inst_dir .= "binary-$architecture";
    }

    # Make sure the files have suitable permissions (*before* installing them)

    if (!chmod($dists_file_mode, $release)) {
        $Error = "Couldn't chmod Release file '$release': $!";
        return 0;
    }

    if (defined($signature) && !(chmod($dists_file_mode, $signature))) {
        $Error = "Couldn't chmod Signature file '$signature': $!";
        return 0;
    }

    # Now install the file(s) into the appropriate place(s).

    if (!rename($release, "$inst_dir/Release")) {
        $Error = "Couldn't install Release file '$release' to '$inst_dir': $!";
        return 0;
    }

    if (defined($signature) && !rename($signature, "$inst_dir/Release.gpg")) {
        $Error = "Couldn't install Signature file '$signature' to '$inst_dir': $!";
        return 0;
    }

    return 1;
}

END {}

1;

__END__

# vim:set tabstop=4 expandtab:
